#include "stdafx.h"
#include "SetupBuilderManager.h"
#include "IPC/IPCCopyData.h"
#include "Registry.h"
#include "IModule.h"
#include "util.h"
#include <shellapi.h>
#include "LauncherManager.h"
#include "ModuleInstance.h"
#include "NaverUtil.h"
#include "HttpClient/httpclient.h"

SetupBuilderManager::SetupBuilderManager()
{

}

SetupBuilderManager::~SetupBuilderManager()
{

}

BOOL SetupBuilderManager::ExtractSourceFile( const TCHAR *source_filename, const TCHAR *target_filename)
{
	const int IDR_APP_SETUP =  137;
	const int IDS_APPSETUP_NAME = 1;
	const int IDS_APPSETUP_KEY = 2;
	HMODULE hModule;
	hModule = LoadLibrary(source_filename);
	if( NULL == hModule )
		return FALSE;

	LoadString( hModule, IDS_APPSETUP_NAME, m_SetupName, 1024 );
	LoadString( hModule, IDS_APPSETUP_KEY, m_SetupKey, 1024 );

	HRSRC hRes = FindResource( hModule, MAKEINTRESOURCE(IDR_APP_SETUP), _T("APP_SETUP") );
	DWORD dwResSize = SizeofResource( hModule, hRes );
	HGLOBAL	hResData = LoadResource(hModule, hRes);
	if (hResData != NULL && dwResSize != 0)
	{
		LPVOID lpData;
		DWORD dwByteWrite;

		lpData = LockResource(hResData);
		HANDLE hFile = CreateFile( target_filename, GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if ( (NULL != hFile) && (INVALID_HANDLE_VALUE != hFile) ) { 
			SetFilePointer(hFile, 0, 0, FILE_BEGIN);
			WriteFile(hFile, (LPBYTE)lpData, dwResSize, &dwByteWrite, NULL);
			if (NULL != hFile)
			{
				CloseHandle(hFile);
			}
			hFile = NULL;
		} else {
			return FALSE;
		}
	}

	FreeLibrary(hModule);
	return TRUE;
}

BOOL SetupBuilderManager::CheckAndNavigateURL(LPCTSTR szAppName)
{
	TCHAR KeyStr[512];
	TCHAR szReadBuffer[MAX_PATH] = {NULL};
	HKEY hRegKey = NULL;
	int  nMaxSize = 1024;
	_tcscpy(szReadBuffer, _T(""));

	wsprintf(KeyStr, _T("Software\\NHN Corporation\\%s"), szAppName);
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, KeyStr, 0, KEY_QUERY_VALUE, &hRegKey) != ERROR_SUCCESS)
		return FALSE;

	if (ERROR_SUCCESS != RegQueryValueEx(hRegKey, _T("NavigateOnce"), 0, 0, (LPBYTE)szReadBuffer, (LPDWORD)&nMaxSize))
	{
		RegCloseKey(hRegKey);
		return FALSE;
	} 

	ShellExecute(NULL, _T("open"), szReadBuffer, _T(""), _T(""), SW_SHOWNORMAL);
	RegCloseKey(hRegKey);

	return TRUE;
}

BOOL SetupBuilderManager::CheckAndSetAutoStart(LPCTSTR szAppName, LPCTSTR szAppKey)
{
	TCHAR KeyStr[512];
	TCHAR szReadBuffer[MAX_PATH] = {NULL};
	HKEY hRegKey = NULL;
	int  nMaxSize = 1024;
	_tcscpy(szReadBuffer, _T(""));

	wsprintf(KeyStr, _T("Software\\NHN Corporation\\%s"), szAppName);
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, KeyStr, 0, KEY_QUERY_VALUE, &hRegKey) != ERROR_SUCCESS)
		return FALSE;

	if (ERROR_SUCCESS != RegQueryValueEx(hRegKey, _T("AutoStartOnce"), 0, 0, (LPBYTE)szReadBuffer, (LPDWORD)&nMaxSize))
	{
		RegCloseKey(hRegKey);
		return FALSE;
	} 
	RegCloseKey(hRegKey);

	if (_tcslen(szReadBuffer) > 0)
	{
		CRegistry reg(HKEY_CURRENT_USER, _T("Software\\Microsoft\\Windows\\CurrentVersion\\Run"));
		reg.SetValue(szAppKey, REG_SZ, (BYTE*)szReadBuffer, _tcslen(szReadBuffer)*sizeof(TCHAR));
	} 
	else
		return FALSE;

	return TRUE;
}

BOOL SetupBuilderManager::CheckAndStartWebPage(LPCTSTR szAppName)
{
	TCHAR KeyStr[512];
	TCHAR szReadBuffer[MAX_PATH] = {NULL};
	HKEY hRegKey = NULL;
	int  nMaxSize = 1024;
	_tcscpy(szReadBuffer, _T(""));

	wsprintf(KeyStr, _T("Software\\NHN Corporation\\%s"), szAppName);
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, KeyStr, 0, KEY_QUERY_VALUE, &hRegKey) != ERROR_SUCCESS)
	{
		if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, KeyStr, 0, KEY_QUERY_VALUE | KEY_WOW64_64KEY, &hRegKey) != ERROR_SUCCESS)
		{
			return FALSE;
		}
	}
	if (ERROR_SUCCESS != RegQueryValueEx(hRegKey, _T("StartWebPageOnce"), 0, 0, (LPBYTE)szReadBuffer, (LPDWORD)&nMaxSize))
	{
		RegCloseKey(hRegKey);
		return FALSE;
	} 
	RegCloseKey(hRegKey);

	CUtil::SetNaverStartPage();

	//if (_tcslen(szReadBuffer) > 0)
	//{
	//	CRegistry reg(HKEY_CURRENT_USER, _T("Software\\Microsoft\\Internet Explorer\\Main"));
	//	reg.SetValue(_T("Start Page"), REG_SZ, (BYTE*)szReadBuffer, _tcslen(szReadBuffer)*sizeof(TCHAR));
	//}
	//else
	//	return FALSE;

	tipbase::GenericHTTPClient httpNClicks;
	httpNClicks.SetNclicks();
	httpNClicks.Request(NULL, _T("http://cc.naver.com/cc?a=ins.home&u=about%3Ablank&nsc=tools.app&m=0"));	

	return TRUE;
}

BOOL SetupBuilderManager::CheckAndPostToolbarProc(LPCTSTR szAppName)
{
	TCHAR KeyStr[512];
	TCHAR szReadBuffer[MAX_PATH] = {NULL};
	HKEY hRegKey = NULL;
	int  nMaxSize = 1024;
	_tcscpy(szReadBuffer, _T(""));

	wsprintf(KeyStr, _T("Software\\NHN Corporation\\%s"), szAppName);
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, KeyStr, 0, KEY_QUERY_VALUE, &hRegKey) != ERROR_SUCCESS)
		return FALSE;

	if (ERROR_SUCCESS != RegQueryValueEx(hRegKey, _T("PostInstOnce"), 0, 0, (LPBYTE)szReadBuffer, (LPDWORD)&nMaxSize))
	{
		RegCloseKey(hRegKey);
		return FALSE;
	} 
	RegCloseKey(hRegKey);


	WIN32_FIND_DATA FindFileData;
	if (INVALID_HANDLE_VALUE != FindFirstFile(szReadBuffer, &FindFileData))
	{
		ShellExecute(NULL, _T("open"), szReadBuffer, _T(""), _T(""), SW_SHOWNORMAL);
	}

	return TRUE;
}

BOOL SetupBuilderManager::CheckAndExecuteApp(LPCTSTR szAppName)
{
	TCHAR KeyStr[512];
	TCHAR szReadBuffer[MAX_PATH] = {NULL};
	HKEY hRegKey = NULL;
	int  nMaxSize = 1024;
	_tcscpy(szReadBuffer, _T(""));

	wsprintf(KeyStr, _T("Software\\NHN Corporation\\%s"), szAppName);
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, KeyStr, 0, KEY_QUERY_VALUE, &hRegKey) != ERROR_SUCCESS)
	{ 
		if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, KeyStr, 0, KEY_QUERY_VALUE | KEY_WOW64_64KEY, &hRegKey) != ERROR_SUCCESS)

		{ 
			return false;
		}
	}
		

	if (ERROR_SUCCESS != RegQueryValueEx(hRegKey, _T("RunOnce"), 0, 0, (LPBYTE)szReadBuffer, (LPDWORD)&nMaxSize))
	{
		RegCloseKey(hRegKey);
		return false;
	} 

	TCHAR *next_token;
	TCHAR * pszToken = _tcstok_s( szReadBuffer, _T("|"), &next_token ); 
	while(pszToken)
	{
		if( 0 != pszToken[0] )
		{
			if( FALSE == ExecProgram(pszToken) )
				return FALSE;
		}
		pszToken = _tcstok_s(NULL, _T("|"), &next_token);
	}

	RegCloseKey(hRegKey);

	return TRUE;
}

// NAVER_SEARCH_ENGINE SETTING.
#include <WtsApi32.h>

/**
* Naver  ⺻ ˻ ϴ Լ 
* @return  ϸ TRUE, ƴϸ FALSE
*/
BOOL SetNaverSearchEngine(LPCTSTR szDisplayName, LPCTSTR szSearchEngineUrl, LPCTSTR szAppName)
{
	// Naver ˻  شǴ  Ѵٸ װɷ ϰ
	// ƴϸ    ְ ̸ ⺻ ˻  
	CRegKey regkey;

	HKEY    hParent = HKEY_CURRENT_USER;
	CString keyName = _T("SOFTWARE\\Microsoft\\Internet Explorer\\SearchScopes");

	if (CheckVista() == TRUE)
	{
		BOOL isVistaUserModeLogin = FALSE;

		IsVistaUserModeLogined(isVistaUserModeLogin);

		if (isVistaUserModeLogin == TRUE)
		{
			CString userSid;
			GetCurrentExplorerUserSID(userSid);

			CString keyNameTemp;
			keyNameTemp.Format(_T("%s\\%s"), userSid, keyName);

			hParent = HKEY_USERS;
			keyName = keyNameTemp;            
		}
	}

	if ( regkey.Open(hParent, keyName ) != ERROR_SUCCESS )
	{
		if ( regkey.Open(hParent, keyName, KEY_WOW64_64KEY ) != ERROR_SUCCESS )
		{
			return FALSE;
		}
	}

	int iIndex = 0;

	while (TRUE)
	{
		TCHAR szProviderGUID[255];
		DWORD dwProviderGUIDLength = sizeof(szProviderGUID);

		LONG ret = regkey.EnumKey(iIndex, szProviderGUID, &dwProviderGUIDLength);
		if ( ret == ERROR_SUCCESS || ret == ERROR_MORE_DATA)
		{
			szProviderGUID[dwProviderGUIDLength] = '\0';

			if (IsNaverProvider(szProviderGUID, szDisplayName, szSearchEngineUrl) == TRUE)
			{
				// ̰ɷ ⺻ ˻  
				if (ERROR_SUCCESS == regkey.SetStringValue( _T("DefaultScope"), szProviderGUID))
				{
					return TRUE;
				}
				else
				{
					return FALSE;
				}
			}

			++iIndex;
		}
		else if (ret == ERROR_NO_MORE_ITEMS)
		{
			break;
		}
		else
		{
			return FALSE;
		}
	}

	// ƿ      װɷ 
	CString strGUID;
	GUID Guid;

	HRESULT hResult = ::CoCreateGuid( &Guid );

	if( SUCCEEDED( hResult ) )
	{
		strGUID.Format(_T("{%.8X-%.4X-%.4X-%.2X%.2X-%.2X%.2X%.2X%.2X%.2X%.2X}"),
			Guid.Data1, Guid.Data2, Guid.Data3,
			Guid.Data4[ 0 ], Guid.Data4[ 1 ], Guid.Data4[ 2 ],
			Guid.Data4[ 3 ], Guid.Data4[ 4 ], Guid.Data4[ 5 ],
			Guid.Data4[ 6 ], Guid.Data4[ 7 ] );
	}

	CRegKey newProviderKey;
	if (newProviderKey.Create(regkey, strGUID) != ERROR_SUCCESS)
	{
		return FALSE;
	}

	//newProviderKey.SetStringValue(_T("DisplayName"), _T("Naver"));
	newProviderKey.SetStringValue(_T("DisplayName"), szDisplayName);
	//newProviderKey.SetStringValue(_T("URL"), _T("http://rd.search.naver.com/search.naver?where=nexearch&sm=ies_hty&ie=utf8&query={searchTerms}"));
	newProviderKey.SetStringValue(_T("URL"), szSearchEngineUrl);

	//IE Version 8  
	int iIEVer = GetIEVersion();
	if ( 8 == iIEVer)
	{
		TCHAR KeyStr[512]={NULL};
		TCHAR szValue[1024] = {NULL};
		memset(szValue, 0, MAX_PATH);
		ULONG  nMaxSize = 1024;
		DWORD dwType = REG_SZ;

#ifdef _NAVER_JAPAN
		wsprintf(KeyStr, _T("Software\\Naver Japan\\%s"), szAppName);	
#else
		wsprintf(KeyStr, _T("Software\\NHN Corporation\\%s"), szAppName);
#endif

		CRegKey rgInstallerInfo;
		if ( ERROR_SUCCESS != rgInstallerInfo.Open(HKEY_LOCAL_MACHINE, KeyStr, KEY_QUERY_VALUE) )
		{
			rgInstallerInfo.Open(HKEY_LOCAL_MACHINE, KeyStr, KEY_QUERY_VALUE | KEY_WOW64_64KEY );
		}

		if (ERROR_SUCCESS == rgInstallerInfo.QueryStringValue(_T("FaviconURL"), szValue, &nMaxSize ))
		{
			newProviderKey.SetStringValue( _T("FaviconURL"), szValue );
		}
		nMaxSize = 1024;
		if (ERROR_SUCCESS == rgInstallerInfo.QueryStringValue(_T("SuggestionsURL_JSON"), szValue, &nMaxSize ))
		{											   
			newProviderKey.SetStringValue( _T("SuggestionsURL_JSON"), szValue );
		}
		nMaxSize = 1024;
		if (ERROR_SUCCESS == rgInstallerInfo.QueryStringValue(_T("OSDFileURL"), szValue, &nMaxSize ))
		{
			newProviderKey.SetStringValue( _T("OSDFileURL"), szValue );
		}

		DWORD dwValue;

		if (ERROR_SUCCESS == rgInstallerInfo.QueryDWORDValue(_T("ShowSearchSuggestions"),dwValue))
		{
			newProviderKey.SetDWORDValue( _T("ShowSearchSuggestions"), dwValue );
		}

		if (ERROR_SUCCESS == rgInstallerInfo.QueryDWORDValue(_T("Codepage"), dwValue))
		{
			newProviderKey.SetDWORDValue( _T("Codepage"), dwValue );
		}


	}



	// ̰ɷ ⺻ ˻  
	if (ERROR_SUCCESS == regkey.SetStringValue( _T("DefaultScope"), strGUID))
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

int GetIEVersion()
{
	CRegKey regkey;
	TCHAR szIEVersion[MAX_PATH];
	ULONG nTCHARs = MAX_PATH;

	if ( regkey.Open(HKEY_LOCAL_MACHINE, _T("SOFTWARE\\Microsoft\\Internet Explorer\\Version Vector"), KEY_READ ) != ERROR_SUCCESS )
	{
		regkey.Close();
		nTCHARs = MAX_PATH;
		// SOFTWARE\\Microsoft\\Internet Explorer\\Version Vector  IE   찡 ֱ  
		// SOFTWARE\\Microsoft\\Internet Explorer  Version  ˾Ƴ ũν üũ ϵ Ѵ.
		if ( regkey.Open(HKEY_LOCAL_MACHINE, _T("SOFTWARE\\Microsoft\\Internet Explorer"), KEY_READ ) != ERROR_SUCCESS )
		{
			return 0;
		}
		if (ERROR_SUCCESS == regkey.QueryStringValue( _T("Version"), szIEVersion, &nTCHARs))
		{
			return  _ttoi( szIEVersion );
		}
		return 0;		
	}

	if (ERROR_SUCCESS == regkey.QueryStringValue( _T("IE"), szIEVersion, &nTCHARs))
	{
		return  _ttoi( szIEVersion );
	}
	else
	{
		regkey.Close();
		nTCHARs = MAX_PATH;
		if ( regkey.Open(HKEY_LOCAL_MACHINE, _T("SOFTWARE\\Microsoft\\Internet Explorer"), KEY_READ ) != ERROR_SUCCESS )
		{
			return 0;
		}
		if (ERROR_SUCCESS == regkey.QueryStringValue( _T("Version"), szIEVersion, &nTCHARs))
		{
			return  _ttoi( szIEVersion );
		}
		return 0;
	}
}

bool CheckVista()
{
	bool bRet = false;
	OSVERSIONINFO osvi;
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

	GetVersionEx (&osvi);

	if ( osvi.dwMajorVersion >= 6 ) 
		bRet = true;
	else
		bRet = false;

	return bRet;
}

/**
* IsNaverSearchEngineConfigured(), SetNaverSearchEngine()  ϴ Լ,  ˻ Provider   Naver  ȮѴ. 
* @return ⺻ ˻ ̸̹ TRUE, ƴϸ FALSE
*/
BOOL IsNaverProvider( LPCTSTR szDefaultScope, LPCTSTR szDisplayName, LPCTSTR szSearchEngineUrl )
{
	if (szDefaultScope == NULL)
	{
		return FALSE;
	}

	CRegKey scopeKey;

	CString searchScopeProvider;
	searchScopeProvider.Format(_T("SOFTWARE\\Microsoft\\Internet Explorer\\SearchScopes\\%s"), szDefaultScope);

	HKEY    hParent = HKEY_CURRENT_USER;
	CString keyName = searchScopeProvider;

	if (CheckVista() == TRUE)
	{
		BOOL isVistaUserModeLogin = FALSE;

		IsVistaUserModeLogined(isVistaUserModeLogin);

		if (isVistaUserModeLogin == TRUE)
		{
			CString userSid;
			GetCurrentExplorerUserSID(userSid);

			CString keyNameTemp;
			keyNameTemp.Format(_T("%s\\%s"), userSid, keyName);

			hParent = HKEY_USERS;
			keyName = keyNameTemp;            
		}
	}

	LONG ret = ERROR_SUCCESS;
	if ((ret = scopeKey.Open(hParent,  keyName, KEY_READ)) != ERROR_SUCCESS )
	{
		if ((ret = scopeKey.Open(hParent,  keyName, KEY_READ | KEY_WOW64_64KEY)) != ERROR_SUCCESS )
		{
			return FALSE;
		}
	}

	TCHAR displayName[255];
	TCHAR searchEngineUrl[MAX_PATH];
	ULONG nTCHARs = sizeof(displayName);
	memset(displayName, 0, 255);
	memset(searchEngineUrl, 0, MAX_PATH);

	if (ERROR_SUCCESS != scopeKey.QueryStringValue( _T("DisplayName"), displayName, &nTCHARs))
	{
		return FALSE;
	}

	nTCHARs = MAX_PATH;
	if (ERROR_SUCCESS != scopeKey.QueryStringValue( _T("URL"), searchEngineUrl, &nTCHARs))
	{
		return FALSE;
	}

	if ((_tcsicmp(displayName, szDisplayName) == 0) && (_tcsicmp(searchEngineUrl, szSearchEngineUrl) == 0) )
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

/**
* ڷ ־ μ ڵ μ ڸ , Ÿ  ȣȴ.
* @param phProcess - ڸ  μ ڵ
* @param sname - ڸ   ( ũ⸦ 300 ´)
* @param bIncDoman -     
* @return ڸ ãµ ̸ TRUE, ƴϸ FALSE
*/ 
BOOL GetProcessUsername(HANDLE *phProcess, TCHAR* sname, BOOL bIncDomain)
{
	HANDLE tok = 0;
	HANDLE hProcess;
	TOKEN_USER *ptu;
	DWORD nlen, dlen;
	TCHAR name[300], dom[300], tubuf[300];
	int iUse;

	BOOL ret = FALSE;

	//if phProcess is NULL we get process handle of this
	//process.
	hProcess = phProcess?*phProcess:GetCurrentProcess();

	//open the processes token
	if (!OpenProcessToken(hProcess,TOKEN_QUERY,&tok)) goto ert;

	//get the SID of the token
	ptu = (TOKEN_USER*)tubuf;
	if (!GetTokenInformation(tok,(TOKEN_INFORMATION_CLASS)1,ptu,300,&nlen)) goto ert;

	//get the account/domain name of the SID
	dlen = 300;
	nlen = 300;
	if (!LookupAccountSid(0, ptu->User.Sid, name, &nlen, dom, &dlen, (PSID_NAME_USE)&iUse)) goto ert;

	//copy info to our static buffer
	if (dlen && bIncDomain) {
		_tcscpy(sname,dom);
		_tcscat(sname,_T(""));
		_tcscat(sname,name);
	} else {
		_tcscpy(sname,name);
	}

	ret = TRUE;

ert:
	if (tok) CloseHandle(tok);
	return ret;
}

/**
* Vista α ڸ  Լ
* @param  userName - α ڸ  (out parameter) 
* @return    TRUE, ƴϸ FALSE
* @remark 9X  ۵ϱ   εϴ ڵ带 ־.
*/
BOOL GetVistaUserLogonName(CString& userName)
{
	BOOL  dwRet = FALSE;
	DWORD dwSessionID = -1; // 0 is 1st console session created on XP, 1 is 1st console session on Vista

	LPTSTR pData = NULL;
	DWORD cbReturned = 0;

	typedef BOOL (WINAPI* FN_ProcessIdToSessionId)(DWORD dwProcessId, DWORD *pSessionId);
	typedef BOOL (WINAPI* FN_WTSQuerySessionInformation)(HANDLE hServer, DWORD SessionId, WTS_INFO_CLASS WTSInfoClass, TCHAR** ppBuffer, DWORD * pBytesReturned);
	typedef VOID (WINAPI* FN_WTSFreeMemory)(PVOID pMemory);

	HMODULE hKernel = NULL;
	HMODULE hWts = NULL;

	FN_ProcessIdToSessionId         pProcessIdToSessionId = NULL;
	FN_WTSQuerySessionInformation  pWTSQuerySessionInformation = NULL;
	FN_WTSFreeMemory                pWTSFreeMemroy = NULL;

	hKernel = LoadLibrary(_T("Kernel32.dll"));
	hWts    = LoadLibrary(_T("WtsApi32.dll"));

	if (hKernel == NULL || hWts == NULL)
	{
		dwRet = FALSE;
		goto exit;
	}

	pProcessIdToSessionId = (FN_ProcessIdToSessionId)GetProcAddress(hKernel, "ProcessIdToSessionId");
	pWTSQuerySessionInformation = (FN_WTSQuerySessionInformation)GetProcAddress(hWts, "WTSQuerySessionInformation");
	pWTSFreeMemroy = (FN_WTSFreeMemory)GetProcAddress(hWts, "WTSFreeMemory");

	if (pProcessIdToSessionId == NULL || pWTSQuerySessionInformation == NULL || pWTSFreeMemroy == NULL)
	{
		dwRet = FALSE;
		goto exit;
	}

	pProcessIdToSessionId(GetCurrentProcessId(), &dwSessionID);

	if( pWTSQuerySessionInformation(WTS_CURRENT_SERVER_HANDLE, dwSessionID, WTSUserName, &pData, &cbReturned) )
	{
		userName = pData;

		dwRet = TRUE;
	}

	pWTSFreeMemroy(pData);

exit:	
	if (hKernel)
	{
		FreeLibrary(hKernel);
		hKernel = NULL;
	}

	if (hWts)
	{
		FreeLibrary(hWts);
		hWts = NULL;
	}

	return dwRet;
}

/**
*  explorer.exe  User SID(Security ID) ´. Ÿ ϰ ں Ʈ   Ѵ.
* @param userSID - SID   
* @return SID   ϸ TRUE, ƴϸ FALSE
* @remark SID   ʱ  static  ĳϴ ڵ尡  ־ Ƽ ȯ濡  ȣ ÿ    ִ.
*         Ƽ  ȯ濡 ȣÿ Լ ϱ Ѵ. 
*/
BOOL GetCurrentExplorerUserSID( CString& userSID)
{
	static CString cachedUserSID;

	if (cachedUserSID.IsEmpty() == FALSE)
	{
		userSID = cachedUserSID;
		return TRUE;
	}

	CTaskManager taskManager;
	CExeModuleInstance *pProcess;

	CString logonUserName;
	GetVistaUserLogonName(logonUserName);

	taskManager.Populate();

	for (unsigned i = 0; i < taskManager.GetProcessCount(); i++)
	{
		pProcess = taskManager.GetProcessByIndex( i );

		CString processName = pProcess->Get_Name();
		processName.MakeLower();

		if (_tcsstr( (LPCTSTR)processName, _T("explorer.exe") ) == 0)
		{
			HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION, FALSE, pProcess->Get_ProcessId());

			if (hProcess == NULL)
			{
				continue;
			}

			TCHAR ownerName[300] = {0,};
			GetProcessUsername(&hProcess, ownerName, FALSE);

			CloseHandle(hProcess);

			if (logonUserName != ownerName)
			{
				continue;
			}

			userSID = pProcess->Get_UserSID();
			cachedUserSID = userSID;

			return TRUE;
		}
	}

	return FALSE;
}

/**
* Vista  α Ǵϴ Լ (  α ƴ ǥ ڷ αߴ Ǵϱ )
* @param isLogin -  α , TRUE   α, FALSE   α (out parameter)
* @return    TRUE, ƴϸ FALSE
* @remark 9X  ۵ϱ   εϴ ڵ带 ־. WTS API ̿
*          α    ڸ Ѵ.   μ     ̵̹Ƿ
*         α ڸ ٸ.  쿡 ǥ   αߴٰ ǴѴ.
*/
BOOL IsVistaUserModeLogined(BOOL& isLogin)
{
	BOOL  dwRet = FALSE;
	DWORD dwSessionID = -1; // 0 is 1st console session created on XP, 1 is 1st console session on Vista

	TCHAR* pData = NULL;
	DWORD cbReturned = 0;

	typedef BOOL (WINAPI* FN_ProcessIdToSessionId)(DWORD dwProcessId, DWORD *pSessionId);
	typedef BOOL (WINAPI* FN_WTSQuerySessionInformation)(HANDLE hServer, DWORD SessionId, WTS_INFO_CLASS WTSInfoClass, TCHAR** ppBuffer, DWORD * pBytesReturned);
	typedef VOID (WINAPI* FN_WTSFreeMemory)(PVOID pMemory);

	HMODULE hKernel = NULL;
	HMODULE hWts = NULL;

	FN_ProcessIdToSessionId         pProcessIdToSessionId = NULL;
	FN_WTSQuerySessionInformation  pWTSQuerySessionInformation = NULL;
	FN_WTSFreeMemory                pWTSFreeMemroy = NULL;

	hKernel = LoadLibrary(_T("Kernel32.dll"));
	hWts    = LoadLibrary(_T("WtsApi32.dll"));

	if (hKernel == NULL || hWts == NULL)
	{
		dwRet = FALSE;
		goto exit;
	}

	pProcessIdToSessionId = (FN_ProcessIdToSessionId)GetProcAddress(hKernel, "ProcessIdToSessionId");
	pWTSQuerySessionInformation = (FN_WTSQuerySessionInformation)GetProcAddress(hWts, "WTSQuerySessionInformation");
	pWTSFreeMemroy = (FN_WTSFreeMemory)GetProcAddress(hWts, "WTSFreeMemory");

	if (pProcessIdToSessionId == NULL || pWTSQuerySessionInformation == NULL || pWTSFreeMemroy == NULL)
	{
		dwRet = FALSE;
		goto exit;
	}

	pProcessIdToSessionId(GetCurrentProcessId(), &dwSessionID);

	if ( pWTSQuerySessionInformation(WTS_CURRENT_SERVER_HANDLE, dwSessionID, WTSUserName, &pData, &cbReturned) )
	{
		TCHAR userName[256 +1];
		DWORD dwData = sizeof(userName);
		GetUserName(userName, &dwData);

		isLogin = (_tcscmp(pData, userName) != 0);

		dwRet = TRUE;
	}

	pWTSFreeMemroy(pData);	 

exit:	
	if (hKernel)
	{
		FreeLibrary(hKernel);
		hKernel = NULL;
	}

	if (hWts)
	{
		FreeLibrary(hWts);
		hWts = NULL;
	}

	return dwRet;
}

bool CheckAndSearchEngine(LPCTSTR szAppName)
{
	TCHAR KeyStr[512];
	TCHAR szDisplayName[255] = {NULL};
	TCHAR szSearchEngineUrl[MAX_PATH] = {NULL};
	memset(szDisplayName, 0, 255);
	memset(szSearchEngineUrl, 0, MAX_PATH);
	HKEY hRegKey = NULL;
	int  nMaxSize = 1024;

	TCHAR szReadBuffer[MAX_PATH] = {NULL};
	_tcscpy(szReadBuffer, _T(""));

#ifdef _NAVER_JAPAN
	wsprintf(KeyStr, _T("Software\\Naver Japan\\%s"), szAppName);	
#else
	wsprintf(KeyStr, _T("Software\\NHN Corporation\\%s"), szAppName);
#endif
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, KeyStr, 0, KEY_QUERY_VALUE, &hRegKey) != ERROR_SUCCESS)
	{
		if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, KeyStr, 0, KEY_QUERY_VALUE | KEY_WOW64_64KEY, &hRegKey) != ERROR_SUCCESS)
			return false;
	}


	if (ERROR_SUCCESS != RegQueryValueEx(hRegKey, _T("StartWebPageOnce"), 0, 0, (LPBYTE)szReadBuffer, (LPDWORD)&nMaxSize))
	{
		RegCloseKey(hRegKey);
		return false;
	} 
	RegCloseKey(hRegKey);

	CUtil::FixInvalidNaverSearchScope();
	BOOL bResult =  CUtil::SetNaverSearchEngine();

	tipbase::GenericHTTPClient httpNClicks;
	httpNClicks.SetNclicks();
	httpNClicks.Request(NULL, _T("http://cc.naver.com/cc?a=ins.search&u=about%3Ablank&nsc=tools.app&m=0"));	

	return true;
}

void SetupBuilderManager::PostScript()
{
	CheckAndExecuteApp(m_SetupName);
	CheckAndNavigateURL(m_SetupName);
	CheckAndSetAutoStart(m_SetupName, m_SetupKey);

	CheckAndStartWebPage(m_SetupName);
	CheckAndSearchEngine(m_SetupName);

	CheckAndPostToolbarProc(m_SetupName);
}
